@@ -204,8 +204,11 @@ run_engine <- function(arm_list,
           input_list_arm <- react_evt(Evt, arm, input_list_arm)
           
           #Get extra objects to be exported
-
-          extra_data <-  mget(inputs_out_v, input_list_arm) 
+          if(is.null(inputs_out_v)){
+            extra_data <- list()
+          } else{
+            extra_data <-  mget(inputs_out_v, input_list_arm) 
+          }
 
           # extra_data <- extra_data[!sapply(extra_data,is.null)]
           extra_data <- extra_data[!vapply(extra_data, is.null, TRUE)]
@@ -216,7 +219,7 @@ run_engine <- function(arm_list,
                                                         arm = arm,
                                                         extra_data
               )
-            
+              
 
         } else {input_list_arm$curtime <- Inf} #if no events, stop
         
